WEBSERVICE = "https://insilicodb.org/app/publicutilities/"
CACHE_DIR = "cache";

#-------------------------------------------------------------------------------
checkHeader = function(header)
{
  if(is.null(header$value())) { error(ERR_NO_CONNECTION); }
    
  if(header$value()['status']==404) { error(ERR_NO_DATA); }
  
  if(header$value()['status']==500) { error(ERR_INTERNAL); }
  
  if(header$value()['status']!=200) { error(ERR_NO_CONNECTION); }

  return(TRUE);
}

#-------------------------------------------------------------------------------
checkBinary = function(data, header)
{
  if(!is.element("binary",header$value()))
  {
    json = fromJSON(rawToChar(data));
    error(json$msg);
  }
  return(TRUE);
}

#-------------------------------------------------------------------------------
getInSilicoJSON = function(action)
{  
  address = pp(WEBSERVICE,action);
  header = basicHeaderGatherer()

  options = curlOptions(followlocation=TRUE,
    ssl.verifypeer=FALSE) # enables redirects
  
  tryCatch({json = getURL(address,
                          headerfunction=header$update,
                          .opts=options,
                          .encoding="UTF-8");},
           error=function(x){ error(ERR_NO_CONNECTION);} );
  
  checkHeader(header);
  
  lst = fromJSON(json);
   
  if(is.element("success",names(lst))) 
	{ 
    if(lst[["success"]]==FALSE) { error(lst[["msg"]]); }
  }
 
  return(lst);
}

#-------------------------------------------------------------------------------
readInSilicoCache = function(gse,gpl,norm,genes,cur)
{
  if(file.access(CACHE_DIR,mode=7)==-1) { dir.create(CACHE_DIR); }
  file = file.path(CACHE_DIR,pp(gse,gpl,norm,genes,"_",cur,".Rdata"));
  if(file.access(file,mode=4)==0)
  {
    msg("Requested study found in local file: ",file);
    return(file);
  }
  else { return(NULL); }
}

#-------------------------------------------------------------------------------
writeInSilicoCache = function(data,gse,gpl,norm,genes,cur)
{
  if(file.access(CACHE_DIR,mode=7)==-1) { dir.create(cache); }
  file = file.path(CACHE_DIR,pp(gse,gpl,norm,genes,"_",cur,".Rdata"));
  fileCon = file(file,"wb");
  writeBin(data,fileCon);
  close(fileCon);
  msg("Requested study stored in local file: ",file);
  return(file);
}

#-------------------------------------------------------------------------------
getInSilicoBinary = function(gse,gpl,norm,genes,cur)
{
  action = pp(WEBSERVICE,"getserie?",
              "dataName=",gse,
              "&dataValues=",genes,
              "&norm=",norm,
              "&jobName=CRDATA",
              "&curation=",cur,
              "&platform=",gpl);
  header=basicHeaderGatherer();
  options = curlOptions(ssl.verifypeer=FALSE)

  tryCatch({data = getBinaryURL(action,headerfunction=header$update,
    .opts=options);},
           error=function(x){ error(ERR_NO_CONNECTION);} );
  
  checkHeader(header);
  checkBinary(data,header);

  return(data);
}

#-------------------------------------------------------------------------------
getInSilicoDataset = function(gse,gpl,norm,genes)
{
  if(genes) { genes = "GENE";  }
  else      { genes = "PROBE"; }	
  
  cur = getInSilicoCuration(gse,gpl);

  file = readInSilicoCache(gse,gpl,norm,genes,cur);
  
  if(is.null(file)) 
  { 
    data = getInSilicoBinary(gse,gpl,norm,genes,cur);
    file = writeInSilicoCache(data,gse,gpl,norm,genes,cur);
  }

  tryCatch({eset = get(load(file))}, 
           error=function(e){error("Corrupted file: ",file);});
  return(eset);
}

#-------------------------------------------------------------------------------
getInSilicoDatasets = function(gse,norm,genes)
{
  platforms = getPlatforms(gse);
  lst = lapply(platforms, function(x){getInSilicoDataset(gse,x,norm,genes);})
  return(lst)
}

#-------------------------------------------------------------------------------
getInSilicoAnnotations = function(gse,gpl)
{
  cur = getInSilicoCuration(gse,gpl);

  action = pp("getannotations?",
              "gse=",gse,
              "&gpl=",gpl,
              "&id=",cur);

  annot = getInSilicoJSON(action);
  return(createPhenoFrame(annot));
} 

#-------------------------------------------------------------------------------
createPhenoFrame = function(annot)
{
  #if no annotations, return empty pData
  if(length(annot)==0) { return(new("AnnotatedDataFrame")); }

  keywords = names(annot[[1]]);
  samples = names(annot);

  ## Warning: all measurements must contain the same annotation terms.
  M = matrix(NA,nrow=length(samples),
                ncol=length(keywords),
                dimnames=list(samples, keywords));

  for(i in samples) 
  {
    M[i,keywords] <- unlist(annot[[i]])[keywords];
  }

  myPhenoFrame = new("AnnotatedDataFrame",
                     data = as.data.frame(M),
                     dimLabels=c("Measurements","Keywords"));
  return(myPhenoFrame);
}
