\name{getDataset}
\alias{getDataset}
\title{Get dataset in ExpressionSet format}
\description{Get dataset for a given series GSE and platform GPL identifier in Bioconductors ExpressionSet format}

\usage{
getDataset(gse, gpl, norm="ORIGINAL", genes=FALSE);
}

\arguments{
\item{gse}{Valid series GSE identifier.}
\item{gpl}{Valid platform GPL identifier.}
\item{norm}{Required preprocessing method. Currently available options are ORIGINAL and FRMA. The ORIGINAL normalization is how the original authors submitted it to Gene Expression Omnibus (GEO). This data is always available and is therefore the default option. The availability of FRMA normalization depends on the availability of the CEL files by the original authors.}
\item{genes}{By default a gene expression matrix containing probes is returned. Precomputed conversions using genes instead of probes are also available if this argument is set to TRUE. This conversion was made using the \code{nsFilter} function from Bioconductors \code{genefilter} package.}
}

\value{
  A Bioconductors ExpressionSet
}

\examples{
\dontrun{
  gse = "GSE4635";
  platforms = getPlatforms(gse);
  eset = getDataset(gse,platforms[[1]],norm="ORIGINAL",genes=FALSE);
  print(dim(eset));
  ## Features  Samples 
  ##    22283        8 
  eset = getDataset(gse,platforms[[1]],norm="ORIGINAL",genes=TRUE);
  print(dim(eset));
  ## Features  Samples 
  ##    12679        8 
}}
