\name{getDatasets}
\alias{getDatasets}
\title{Get datasets in ExpressionSet format}
\description{Get datasets (a 'series' object in Gene Expression Omnibus terminology) for a given series GSE identifier for every platform it contains}

\usage{
getDatasets(gse, norm="ORIGINAL", genes=FALSE);
}

\arguments{
\item{gse}{Valid series GSE identifier.}
\item{norm}{Required preprocessing method. Currently available options are ORIGINAL and FRMA. The ORIGINAL normalization is how the original authors submitted it to Gene Expression Omnibus (GEO). This data is always available and is therefore the default option. The availability of FRMA normalization depends on the availability of the CEL files by the original authors.}
\item{genes}{By default a gene expression matrix containing probes is returned. Precomputed conversions using genes instead of probes are also available if this argument is set to TRUE. This conversion was made using the \code{nsFilter} function from Bioconductors \code{genefilter} package.}
}

\value{
  A list with a Bioconductors ExpressionSet for every platform
}

\examples{
\dontrun{
  getPlatforms("GSE781");
  ## [1] "GPL96" "GPL97"
  esets = getDatasets("GSE781");
  sapply(esets, annotation)
  ## [1] "hgu133a" "hgu133b"
}}
